using System;
using System.Text;
//using org.bouncycastle.math;
//using org.bouncycastle.math.ec;
//using org.bouncycastle.security;
using org.bouncycastle.crypto;
//using org.bouncycastle.crypto.signers;
using org.bouncycastle.crypto.digests;
//using org.bouncycastle.crypto.engines;
//using org.bouncycastle.crypto.agreement;
//using org.bouncycastle.crypto.generators;
//using org.bouncycastle.crypto.parameters;
//using org.bouncycastle.crypto.macs;
//using org.bouncycastle.crypto.modes;
//using org.bouncycastle.crypto.paddings;
using org.bouncycastle.util.encoders;
using org.bouncycastle.util.test;


namespace test.crypto
{
	/**
	 * standard vector test for MD4 from RFC 1320.
	 */
	public class MD4DigestTest: Test
	{
		static private String  testVec1 = "";
		static private String  resVec1 = "31d6cfe0d16ae931b73c59d7e0c089c0";

		static private String  testVec2 = "61";
		static private String  resVec2 = "bde52cb31de33e46245e05fbdbd6fb24";

		static private String  testVec3 = "616263";
		static private String  resVec3 = "a448017aaf21d8525fc10ae87aa6729d";

		static private String  testVec4 = "3132333435363738393031323334353637383930313233343536373839303132333435363738393031323334353637383930313233343536373839303132333435363738393031323334353637383930";
		static private String  resVec4 = "e33b4ddc9c38f2199c3e7b164fcc0536";

		public String getName()
		{
			return "MD4";
		}

		public TestResult perform()
		{
			Digest  digest = new MD4Digest();
			byte[]  resBuf = new byte[digest.getDigestSize()];
			String  resStr;

			//
			// test 1
			//
			digest.doFinal(resBuf, 0);

			resStr = ByteArray2String(Hex.encode(resBuf));
			if (!resVec1.Equals(resStr))
			{
				return new SimpleTestResult(false,
					"MD4 failing standard vector test 1"
					+ System.Environment.NewLine
					+ "    expected: " + resVec1
					+ System.Environment.NewLine
					+ "    got     : " + resStr);
			}

			//
			// test 2
			//
			byte[]  bytes = Hex.decode(testVec2);

			digest.update(bytes, 0, bytes.Length);

			digest.doFinal(resBuf, 0);

			resStr = ByteArray2String(Hex.encode(resBuf));
			if (!resVec2.Equals(resStr))
			{
				return new SimpleTestResult(false,
					"MD4 failing standard vector test 2"
					+ System.Environment.NewLine
					+ "    expected: " + resVec2
					+ System.Environment.NewLine
					+ "    got     : " + resStr);
			}

			//
			// test 3
			//
			bytes = Hex.decode(testVec3);

			digest.update(bytes, 0, bytes.Length);

			digest.doFinal(resBuf, 0);

			resStr = ByteArray2String(Hex.encode(resBuf));
			if (!resVec3.Equals(resStr))
			{
				return new SimpleTestResult(false,
					"MD4 failing standard vector test 3"
					+ System.Environment.NewLine
					+ "    expected: " + resVec3
					+ System.Environment.NewLine
					+ "    got     : " + resStr);
			}

			//
			// test 4
			//
			bytes = Hex.decode(testVec4);

			digest.update(bytes, 0, bytes.Length);

			digest.doFinal(resBuf, 0);

			resStr = ByteArray2String(Hex.encode(resBuf));
			if (!resVec4.Equals(resStr))
			{
				return new SimpleTestResult(false,
					"MD4 failing standard vector test 4"
					+ System.Environment.NewLine
					+ "    expected: " + resVec4
					+ System.Environment.NewLine
					+ "    got     : " + resStr);
			}

			//
			// test 5
			//
			bytes = Hex.decode(testVec4);

			digest.update(bytes, 0, bytes.Length/2);

			// clone the Digest
			Digest d = new MD4Digest((MD4Digest)digest);

			digest.update(bytes, bytes.Length/2, bytes.Length - bytes.Length/2);
			digest.doFinal(resBuf, 0);

			resStr = ByteArray2String(Hex.encode(resBuf));
			if (!resVec4.Equals(resStr))
			{
				return new SimpleTestResult(false,
					"MD4 failing standard vector test 5"
					+ System.Environment.NewLine
					+ "    expected: " + resVec4
					+ System.Environment.NewLine
					+ "    got     : " + resStr);
			}

			d.update(bytes, bytes.Length/2, bytes.Length - bytes.Length/2);
			d.doFinal(resBuf, 0);

			resStr = ByteArray2String(Hex.encode(resBuf));
			if (!resVec4.Equals(resStr))
			{
				return new SimpleTestResult(false,
					"MD4 failing standard vector test 5"
					+ System.Environment.NewLine
					+ "    expected: " + resVec4
					+ System.Environment.NewLine
					+ "    got     : " + resStr);
			}
			return new SimpleTestResult(true, getName() + ": Okay");
		}
		private String ByteArray2String(byte[] bs)
		{
			StringBuilder s = new StringBuilder();
			for (int i = 0; i < bs.Length; i++) 
			{
				s.Append(Convert.ToChar(bs[i]));
			}
			return s.ToString();
		}
		[STAThread]
		static void Main(string[] args)
		{
			MD4DigestTest   test = new MD4DigestTest();
			TestResult      result = test.perform();

			Console.WriteLine(result.ToString());
		}
	}

}
