using System;
using System.IO;
using System.Collections;
//using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	/**
	* ASN.1 TaggedObject - in ASN.1 nottation this is any object proceeded by
	* a [n] where n is some number - these are assume to follow the construction
	* rules (as with sequences).
	*/
	public abstract class ASN1TaggedObject : DERObject
	{
		internal int             tagNo;
		internal bool         empty = false;
		internal bool         flagExplicit = true;
		internal DEREncodable    obj = null;

		static public ASN1TaggedObject getInstance(
			ASN1TaggedObject    obj,
			bool             flagExplicit)
		{
			if (flagExplicit)
			{
				return (ASN1TaggedObject)obj.getObject();
			}

			throw new ArgumentException("implicitly tagged tagged object");
		}

		/**
		* @param tagNo the tag number for this object.
		* @param obj the tagged object.
		*/
		public ASN1TaggedObject(
			int             tagNo,
			DEREncodable    obj)
		{
			this.flagExplicit = true;
			this.tagNo = tagNo;
			this.obj = obj;
		}

		/**
		* @param flagExplicit true if the object is explicitly tagged.
		* @param tagNo the tag number for this object.
		* @param obj the tagged object.
		*/
		public ASN1TaggedObject(
			bool         flagExplicit,
			int             tagNo,
			DEREncodable    obj)
		{
			this.flagExplicit = flagExplicit;
			this.tagNo = tagNo;
			this.obj = obj;
		}
		
		public override  bool Equals(
			Object o)
		{
			if ((o == null) || !(typeof(ASN1TaggedObject).IsInstanceOfType(o)))
			{
				return false;
			}
	        
			ASN1TaggedObject other = (ASN1TaggedObject)o;
	        
			if(tagNo != other.tagNo || empty != other.empty || flagExplicit != other.flagExplicit)
			{
				return false;
			}
			
			if(obj == null)
			{
				if(other.obj != null)
				{
					return false;
				}
			}
			else
			{
				if(!(obj.Equals(other.obj)))
				{
					return false;
				}
			}
			
			return true;
		}

		public override int GetHashCode()
		{
			return base.GetHashCode();
		}
		
		public int getTagNo()
		{
			return tagNo;
		}

		/**
		* return whether or not the object may be explicitly tagged. 
		* <p>
		* Note: if the object has been read from an input stream, the only
		* time you can be sure if isExplicit is returning the true state of
		* affairs is if it returns false. An implicitly tagged object may appear
		* to be explicitly tagged, so you need to understand the context under
		* which the reading was done as well, see getObject below.
		*/
		public bool isExplicit()
		{
			return flagExplicit;
		}

		public bool isEmpty()
		{
			return empty;
		}

		/**
		* return whatever was following the tag.
		* <p>
		* Note: tagged objects are generally context dependent if you're
		* trying to extract a tagged object you should be going via the
		* appropriate getInstance method.
		*/
		public DERObject getObject()
		{
			if (obj != null)
			{
				return obj.getDERObject();
			}

			return null;
		}
	}

}