using System;
using System.IO;
using System.Collections;
using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	public class BERConstructedOctetString : DEROctetString
	{
		/**
		* convert a vector of octet strings into a single byte string
		*/
		static private byte[] toBytes(ArrayList  octs)
		{
			MemoryStream   bOut = new MemoryStream();

			for (int i = 0; i != octs.Count; i++)
			{
				DEROctetString  o = (DEROctetString)octs[i];

				try
				{
					byte[] tmp = o.getOctets();
					bOut.Write(tmp, 0, tmp.Length);
				}
				catch (IOException e)
				{
					throw new Exception("exception converting octets " + e.ToString());
				}
			}

			return bOut.ToArray();
		}

		private ArrayList  octs;

		/**
		* @param in_string the octets making up the octet string.
		*/
		public BERConstructedOctetString(byte[]  in_string) : base(in_string){}


		public BERConstructedOctetString(ArrayList  octs) : base(toBytes(octs))
		{
			this.octs = octs;
		}

		public BERConstructedOctetString(
			DERObject  obj) : base(obj) {}

		public BERConstructedOctetString(
			DEREncodable  obj) : base(obj.getDERObject()) {}


		public override  byte[] getOctets()
		{
			return m_string;
		}

		/**
		* return the DER octets that make up this string.
		*/
		public IEnumerator getObjects()
		{
			if (octs == null)
			{
				octs = generateOcts();
			}

			return octs.GetEnumerator();
		}

		private ArrayList generateOcts()
		{
			int     start = 0;
			int     end = 0;
			ArrayList  vec = new ArrayList();

			while ((end + 1) < m_string.Length)
			{
				if (m_string[end] == 0 && m_string[end + 1] == 0)
				{
					byte[]  nStr = new byte[end - start + 1];

					for (int i = 0; i != nStr.Length; i++)
					{
						nStr[i] = m_string[start + i];
					}

					vec.Add(new DEROctetString(nStr));
					start = end + 1;
				}
				end++;
			}

			byte[]  nStr2 = new byte[m_string.Length - start];
			for (int i = 0; i != nStr2.Length; i++)
			{
				nStr2[i] = m_string[start + i];
			}

			vec.Add(new DEROctetString(nStr2));

			return vec;
		}

		public override  void encode(DEROutputStream outStream)
			//throws IOException
		{
			if (typeof(ASN1OutputStream).IsInstanceOfType(outStream) ||
				typeof(BEROutputStream).IsInstanceOfType(outStream))
			{
				outStream.write(DERTags.CONSTRUCTED | DERTags.OCTET_STRING);

				outStream.write(0x80);

				if (octs == null)
				{
					octs = generateOcts();
				}

				for (int i = 0; i != octs.Count; i++)
				{
					outStream.writeObject(octs[i]);
				}

				outStream.write(0x00);
				outStream.write(0x00);
			}
			else
			{
				base.encode(outStream);
			}
		}
	}

}