
using System;
using System.Collections;
//using org.bouncycastle.io;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.x509
{
	public class CertificatePolicies: DEREncodable
	{
		static readonly DERObjectIdentifier anyPolicy = new DERObjectIdentifier("2.5.29.32.0");

		ArrayList policies = new ArrayList();

	/**
	* @deprecated use an ASN1Sequence of PolicyInformation
	*/
		public static CertificatePolicies getInstance(
			ASN1TaggedObject obj,
			bool flagExplicit)
		{
			return getInstance(ASN1Sequence.getInstance(obj, flagExplicit));
		}

	/**
	* @deprecated use an ASN1Sequence of PolicyInformation
	*/
		public static CertificatePolicies getInstance(
			Object  obj)
		{
			
			if (typeof(CertificatePolicies).IsInstanceOfType(obj))
			{
				return (CertificatePolicies)obj;
			}
			else if (typeof(ASN1Sequence).IsInstanceOfType(obj))
			{
				return new CertificatePolicies((ASN1Sequence)obj);
			}

			throw new ArgumentException("unknown object in factory");
		}

	/**
	* @deprecated use an ASN1Sequence of PolicyInformation
	*/
		public CertificatePolicies(
			ASN1Sequence   seq)
		{
			IEnumerator e = seq.getObjects();
			while (e.MoveNext())
			{
				ASN1Sequence s = (ASN1Sequence)e.Current;
				policies.Add(s.getObjectAt(0));
			}
			// For now we just don't handle PolicyQualifiers
		}

		/**
		* create a certificate policy with the given OID.
		* @deprecated use an ASN1Sequence of PolicyInformation
		*/
		public CertificatePolicies(
			DERObjectIdentifier p)
		{
			policies.Add(p);
		}

		/**
		* create a certificate policy with the policy given by the OID represented
		* by the string p.
		* @deprecated use an ASN1Sequence of PolicyInformation
		*/
		public CertificatePolicies(
			String p): this(new DERObjectIdentifier(p)) {}

		public void addPolicy(
			String p)
		{
			policies.Add(new DERObjectIdentifier(p));
		}

		public String getPolicy(int nr)
		{
			if (policies.Count > nr)
				return ((DERObjectIdentifier)policies[nr]).getId();

			return null;
		}

		/**
		* <pre>
		* certificatePolicies ::= SEQUENCE SIZE (1..MAX) OF PolicyInformation
		*
		* PolicyInformation ::= SEQUENCE {
		*   policyIdentifier   CertPolicyId,
		*   policyQualifiers   SEQUENCE SIZE (1..MAX) OF
		*                           PolicyQualifierInfo OPTIONAL }
		*
		* CertPolicyId ::= OBJECT IDENTIFIER
		*
		* PolicyQualifierInfo ::= SEQUENCE {
		*   policyQualifierId  PolicyQualifierId,
		*   qualifier          ANY DEFINED BY policyQualifierId }
		*
		* PolicyQualifierId ::=
		*   OBJECT IDENTIFIER ( id-qt-cps | id-qt-unotice )
		* </pre>
		* @deprecated use an ASN1Sequence of PolicyInformation
		*/
		public DERObject getDERObject()
		{
			ASN1EncodableVector  v = new ASN1EncodableVector();

			// We only do policyIdentifier yet...
			for (int i=0;i<policies.Count;i++)
			{
				v.add(new DERSequence((DERObjectIdentifier)policies[i]));
			}

			return new DERSequence(v);
		}

		public String toString()
		{
			String p = null;
			for (int i=0;i<policies.Count;i++)
			{
				if (p != null)
					p += ", ";
				p += ((DERObjectIdentifier)policies[i]).getId();
			}
			return "CertificatePolicies: "+p;
		}
	}

}