using System;
//using System.Collections;
//using org.bouncycastle.io;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.x509
{
	/**
	 * <pre>
	 *    id-ce-keyUsage OBJECT IDENTIFIER ::=  { id-ce 15 }
	 *
	 *    KeyUsage ::= BIT STRING {
	 *         digitalSignature        (0),
	 *         nonRepudiation          (1),
	 *         keyEncipherment         (2),
	 *         dataEncipherment        (3),
	 *         keyAgreement            (4),
	 *         keyCertSign             (5),
	 *         cRLSign                 (6),
	 *         encipherOnly            (7),
	 *         decipherOnly            (8) }
	 * </pre>
	 */
	public class KeyUsage: DERBitString
	{
		public static readonly int        digitalSignature = (1 << 7); 
		public static readonly int        nonRepudiation   = (1 << 6);
		public static readonly int        keyEncipherment  = (1 << 5);
		public static readonly int        dataEncipherment = (1 << 4);
		public static readonly int        keyAgreement     = (1 << 3);
		public static readonly int        keyCertSign      = (1 << 2);
		public static readonly int        cRLSign          = (1 << 1);
		public static readonly int        encipherOnly     = (1 << 0);
		public static readonly int        decipherOnly     = (1 << 15);

		/**
		 * Basic constructor.
		 * 
		 * @param usage - the bitwise OR of the Key Usage flags giving the
		 * allowed uses for the key.
		 * e.g. (X509KeyUsage.keyEncipherment | X509KeyUsage.dataEncipherment)
		 */
		public KeyUsage(
			int usage):  base(getBytes(usage), getPadBits(usage)){}

		public KeyUsage(
			DERBitString usage) : base(usage.getBytes(), usage.getPadBits()){}

		public override String ToString()
		{
			return "KeyUsage: 0x" + (data[0] & 0xff).ToString("X");
		}
	}

}