
using System;
//using System.Collections;
using org.bouncycastle.crypto;
using org.bouncycastle.crypto.digests;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.x509
{
	/**
	* <pre>
	* SubjectKeyIdentifier::= OCTET STRING
	* </pre>
	*/
	public class SubjectKeyIdentifier: DEREncodable
	{
		private byte[] keyidentifier;

		public static SubjectKeyIdentifier getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(ASN1OctetString.getInstance(obj, flagExplicit));
		}

		public static SubjectKeyIdentifier getInstance(
			Object obj)
		{
			if ((obj == null) || (typeof(SubjectKeyIdentifier).IsInstanceOfType(obj)))
			{
				return (SubjectKeyIdentifier)obj;
			}
	        
			if (typeof(SubjectPublicKeyInfo).IsInstanceOfType(obj))
			{
				return new SubjectKeyIdentifier((SubjectPublicKeyInfo)obj);
			}
	        
			if (typeof(ASN1OctetString).IsInstanceOfType(obj))
			{
				return new SubjectKeyIdentifier((ASN1OctetString)obj);
			}
	        
			throw new ArgumentException("Invalid SubjectKeyIdentifier: " + obj.GetType().ToString());
		}
		
		public SubjectKeyIdentifier(
			byte[] keyid)
		{
			this.keyidentifier=keyid;
		}

		public SubjectKeyIdentifier(
			ASN1OctetString  keyid)
		{
			this.keyidentifier=keyid.getOctets();

		}

		/**
		*
		* Calulates the keyidentifier using a SHA1 hash over the BIT STRING
		* from SubjectPublicKeyInfo as defined in RFC2459.
		*
		**/
		public SubjectKeyIdentifier(
			SubjectPublicKeyInfo    spki)
		{
			Digest  digest = new SHA1Digest();
			byte[]  resBuf = new byte[digest.getDigestSize()];

			byte[] bytes = spki.getPublicKeyData().getBytes();
			digest.update(bytes, 0, bytes.Length);
			digest.doFinal(resBuf, 0);
			this.keyidentifier=resBuf;
		}

		public byte[] getKeyIdentifier()
		{
			return keyidentifier;
		}

		/**
		* <pre>
		* SubjectKeyIdentifier := OCTET STRING
		* </pre>
		*/
		public DERObject getDERObject()
		{
			return new DEROctetString(keyidentifier);
		}
	}

}